module.exports = {
  arrowParens: 'avoid',
  bracketSpacing: false,
  printWidth: 120,
  quoteProps: 'consistent',
  semi: true,
  singleQuote: true,
  jsxSingleQuote: true,
  tabWidth: 2,
  trailingComma: 'all',
  plugins: ['prettier-plugin-organize-imports'],
};
